//
//  GameViewController.swift
//  My3DGame
//
//  Created by 杉本裕樹 on 2017/05/14.
//  Copyright © 2017年 name. All rights reserved.
//

import UIKit
import QuartzCore
import SceneKit
import GameplayKit

class GameViewController: UIViewController {
    let agentSystem = GKComponentSystem(componentClass: GKAgent3D.self)
    var prevTime: TimeInterval = 0
    let shipAgent = GKAgent3D()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let scnView = view as? SCNView
        scnView?.delegate = self
        scnView?.isPlaying = true
        let scene = SCNScene(named: "art.scnassets/ship.scn")
        scnView?.scene = scene
        
        // 光源の設置
        let lightNode = SCNNode()
        lightNode.light = SCNLight()
        lightNode.light?.type = .omni
        lightNode.position = SCNVector3(x: 0, y: 10, z: 10)
        scene?.rootNode.addChildNode(lightNode)
        let ambientLightNode = SCNNode()
        ambientLightNode.light = SCNLight()
        ambientLightNode.light?.type = .ambient
        ambientLightNode.light?.color = UIColor.darkGray
        scene?.rootNode.addChildNode(ambientLightNode)
        
        // 飛行機の角度調整
        let ship = scene?.rootNode.childNode(withName: "ship", recursively: true)
        ship?.eulerAngles = SCNVector3(x: 0, y: Float.pi, z: 0)
        
        let targetAgent = GKAgent3D()
        targetAgent.position = float3(0, 0, -300)
        shipAgent.maxAcceleration = 1
        shipAgent.maxSpeed = 1
        shipAgent.delegate = self
        let obstacle = GKSphereObstacle(radius: 100)
        obstacle.position = vector_float3(0, 0, -150)
        shipAgent.behavior = GKBehavior(goals: [
            GKGoal(toSeekAgent: targetAgent),
            GKGoal(toAvoid: [obstacle], maxPredictionTime: 100.0)
            ], andWeights: [NSNumber(value: 1), NSNumber(value: 50)])
        agentSystem.addComponent(shipAgent)
    }
    
    override var prefersStatusBarHidden: Bool {
        return true
    }
}

extension GameViewController: SCNSceneRendererDelegate {
    // 1フレーム毎に呼ばれるメソッド
    func renderer(_ renderer: SCNSceneRenderer, updateAtTime time: TimeInterval) {
        if prevTime == 0 {
            prevTime = time
        }
        agentSystem.update(deltaTime: time - prevTime)
    }
}

extension GameViewController: GKAgentDelegate {
    func agentDidUpdate(_ agent: GKAgent) {
        if let ship = (view as? SCNView)?.scene?.rootNode.childNode(withName: "ship", recursively: true), let agent = agent as? GKAgent3D {
            ship.position = SCNVector3(agent.position)
        }
    }
}
